import _extends from "@babel/runtime/helpers/esm/extends";
/**
 * transformEventsListenersIntoCallbackMap: transform event listeners list
 * into a callback map, where id is EVENT_TYPE, and value are event type callbacks list
 * 
eventListeners = [
  {
    eventNames: [EVENT_TYPE_1, EVENT_TYPE_2],
    listener: listener1Fn
  },
  {
    eventNames: [EVENT_TYPE_1],
    listener: listener2Fn
  },
  {
    eventNames: [EVENT_TYPE_3],
    listener: listener3Fn
  }
]
 
callbackMap: {
  EVENT_TYPE_1: [listener1Fn, listener2Fn],
  EVENT_TYPE_2: [listener1Fn]
  EVENT_TYPE_3: [listener3Fn]
}
 */
export var transformEventsListenersIntoCallbackMap = function transformEventsListenersIntoCallbackMap(eventListeners) {
  var eventListenerToCallbackMap = function eventListenerToCallbackMap(acc, _ref) {
    var eventNames = _ref.eventNames,
      callback = _ref.listener;
    return eventNames.reduce(addCallbackToMap(callback), acc);
  };
  var addCallbackToMap = function addCallbackToMap(callback) {
    return function (acc, name) {
      var _extends2;
      return _extends({}, acc, (_extends2 = {}, _extends2[name] = [callback].concat(acc[name] || []), _extends2));
    };
  };
  var transformListeners = function transformListeners(listeners) {
    return listeners.reduce(eventListenerToCallbackMap, {});
  };
  return transformListeners(eventListeners);
};